/*
 * Copyright (C) 2010 Andrew Kozin AKA Stanson <stanson@pangolin.ru>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dsp_open(), dsp_play() and dsp_close() functions for OSS
 */

#include <sys/soundcard.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>

#include "log.h"
#include "audio.h"

/* default sound device */
#define SND_DEVICE		"/dev/dsp"

/* OSS buffer size */
#define FRAGMENT_BITS 10
#define NB_FRAGMENTS  ( 32768 >> FRAGMENT_BITS )
#define NB_SAMPLES ( ( 1 << FRAGMENT_BITS ) / PCM_SAMPLE_SIZE )

static int dsp = -1;

static int dsp_ioctl( req, p ) { return ioctl( dsp, req, &p ); }

int dsp_open( char * name, int verbose )
{
  if( ( dsp = open( name[0] ? name : SND_DEVICE, O_RDWR ) ) < 0 )
  {
    _logf( LOG_ERR, "Can't open %s!\n", name );
    return dsp;
  }

  if( dsp_ioctl( SNDCTL_DSP_SETFRAGMENT, (NB_FRAGMENTS << 16) | FRAGMENT_BITS ) == -1 ) goto err;
  if( dsp_ioctl( SNDCTL_DSP_SETFMT,      AFMT_S16_LE ) == -1 ) goto err;
  if( dsp_ioctl( SNDCTL_DSP_STEREO,      0 ) == -1 ) goto err;
  if( dsp_ioctl( SNDCTL_DSP_SPEED,       PCM_SAMPLE_RATE ) == -1 ) goto err;
  return dsp;

err:
  close( dsp );
  dsp = -1;
  _logf( LOG_ERR, "Can't ioctl %s!\n", name );
  return dsp;
}

int dsp_play( short * data, int len ) { return write( dsp, data, len ); }

void dsp_close( void ) { if( dsp >= 0 ) close( dsp ); }
