/*
 * Copyright (C) 2010 Andrew Kozin AKA Stanson <stanson@pangolin.ru>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>

#include "conf.h"

void conf_set_val( struct _conf_params * p, char * s, char end )
{
    char	*d = p->v_str;

    switch( p->type )
    {
	case 'b': *(p->v_bool)  = strncmp( "yes" , s, 3 ) ? 0 : 1; break;
	case 'u': *(p->v_ulong) = strtoul( s, NULL, 0 ); break;
	case 'l': *(p->v_long)  = strtol( s, NULL, 0 ); break;
	case 's': while( *s && *s != '\n' && *s != end ) *d++ = *s++; *d = 0; break;
    }
}

void conf_read_val( char * str, struct _conf_params * params )
{
    char	end = ' ';
    char	*s = strstr( str, params->name );

    if(s)
    {
	s += strlen( params->name );
	while( *s && isspace( *s ) ) s++;
	if( *s && *s != '\n' )
	{
	    if( *s == '"' )  { s++; end = '"'; }
	    if( *s == '\'' ) { s++; end = '\''; }
	    conf_set_val( params, s, end );
	}
    }
}

void conf_set_defaults( struct _conf_params * params )
{
    struct _conf_params	*p;

    for( p = params; p->name; p++ )
	conf_set_val( p, p->d_val, 0 );
}

void conf_parse( char * conf_name, struct _conf_params * params )
{
    FILE		*stream;
    char		str[4096];
    struct _conf_params	*p;

    conf_set_defaults( params );

    stream = fopen( conf_name, "r" );

    if( stream )
    {
	while( fgets( str, sizeof(str), stream ) )
	{
	    if(str[0] == '\n' || str[0] == '#') continue;
	    for( p = params; p->name; p++ )
		conf_read_val( str, p );
	}
	fclose(stream);
    }
}
