/*
 * Copyright (C) 2010 Andrew Kozin AKA Stanson <stanson@pangolin.ru>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <bluetooth/bluetooth.h>
#include <bluetooth/hci.h>

#include "log.h"

int hci_get_local( char * name, char * addr )
{
    int				i, ret = -1, sock;
    struct hci_dev_info		di;
    struct hci_dev_list_req	*dl;
    struct hci_dev_req		*dr;

    if( !name || !addr ) return ret;

    dl = malloc( HCI_MAX_DEV * sizeof(struct hci_dev_req) + sizeof(uint16_t) );
    if( !dl )
    {
	_logf( LOG_ERR, "Out of memory!\n" );
	return ret;
    }

    sock = socket( AF_BLUETOOTH, SOCK_RAW, BTPROTO_HCI );
    if( sock < 0 )
    {
	_logf( LOG_ERR, "HCI socket() failed: %s\n", strerror( errno ) );
	goto out;
    }

    dl->dev_num = HCI_MAX_DEV;
    dr = dl->dev_req;
    ret = ioctl( sock, HCIGETDEVLIST, (void *) dl );
    if( ret < 0 )
    {
	_logf( LOG_ERR, "HCI ioctl(,HCIGETDEVLIST,) failed: %s\n", strerror( errno ) );
	goto out_close;
    }

    for( i = 0; i< dl->dev_num; i++ )
    {
	di.dev_id = (dr+i)->dev_id;

	ret = ioctl( sock, HCIGETDEVINFO, (void *) &di );
	if( ret < 0 ) continue;

	if( bacmp( &di.bdaddr, BDADDR_ANY ) )
	{
	    if( !name[0] || !strcmp( di.name, name ) )
	    {
		ba2str( &di.bdaddr, addr );
		strcpy( name, di.name );
		goto out_close;
	    }
	}
    }

    // nothing found
    ret = -1;

out_close:
    close( sock );
out:
    free( dl );
    return ret;
}
