/*
 * Copyright (C) 2010 Andrew Kozin AKA Stanson <stanson@pangolin.ru>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <errno.h>
#include <unistd.h>
#include <fcntl.h>

#include "sco.h"
#include "log.h"

int sco_start_server( const char * local )
{
    int			sockfd, ret, flags;
    struct sockaddr_sco	src;

    sockfd = socket( PF_BLUETOOTH, SOCK_SEQPACKET, BTPROTO_SCO );
    if( sockfd < 0 )
    {
	_logf( LOG_ERR, "SCO socket() failed: %s\n", strerror( errno ) );
	return sockfd;
    }

    memset( &src, 0, sizeof(src) );
    src.sco_family = AF_BLUETOOTH;
    str2ba( local, &src.sco_bdaddr );

    ret = bind( sockfd, (struct sockaddr *) &src, sizeof(src) );
    if( ret < 0 )
    {
	_logf( LOG_ERR, "SCO bind() failed: %s\n", strerror( errno ) );
	close( sockfd );
	return -1;
    }

    flags = fcntl( sockfd, F_GETFL );
    flags |= O_NONBLOCK;
    ret = fcntl( sockfd, F_SETFL, flags );
    if( ret < 0 )
    {
	_logf( LOG_ERR, "SCO fcntl() failed: %s\n", strerror( errno ) );
	close( sockfd );
	return -1;
    }

    ret = listen( sockfd, 5 );
    if( ret < 0 )
    {
	_logf( LOG_ERR, "SCO listen() failed: %s\n", strerror( errno ) );
	close( sockfd );
	return -1;
    }

    return sockfd;
}

int sco_set_mtu( int fd, uint16_t mtu )
{
    int			ret;
    struct sco_options	sco_opt;
    socklen_t		len;

    len = sizeof(sco_opt);
    memset( &sco_opt, 0, len );
    ret = getsockopt( fd, SOL_SCO, SCO_OPTIONS, &sco_opt, &len );
    if( ret < 0 )
    {
	_logf( LOG_ERR, "SCO getsockopt() failed: %s\n", strerror( errno ) );
	return ret;
    }

    sco_opt.mtu = mtu;

    ret = setsockopt( fd, SOL_SCO, SCO_OPTIONS, &sco_opt, len );
    if( ret < 0 )
    {
	_logf( LOG_ERR, "SCO setsockopt() failed: %s\n", strerror( errno ) );
	return ret;
    }

    return ret;
}
